
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.paragraph.interfaces.contact import IContactParagraph
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IBaseParagraph
from pyams_content.component.paragraph.interfaces.contact import IContactParagraph as IContactParagraphBase
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer
from transaction.interfaces import ITransactionManager
from z3c.form.interfaces import INPUT_MODE

# import packages
from onf_website.component.paragraph.contact import ContactParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm
from pyams_content.component.paragraph.zmi.contact import ContactParagraphAddForm as BaseContactParagraphAddForm, \
    ContactParagraphPropertiesEditForm as BaseContactParagraphPropertiesEditForm
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer


@pagelet_config(name='add-contact-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ContactParagraphAddForm(BaseContactParagraphAddForm):
    """Contact paragraph add form"""

    fields = field.Fields(IContactParagraph).select('title', 'name', 'charge', 'photo', 'gps_location', 'city',
                                                    'address', 'renderer')

    def create(self, data):
        return ContactParagraph()


@view_config(name='add-contact-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphAJAXAddForm(BaseParagraphAJAXAddForm, ContactParagraphAddForm):
    """Contact paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IContactParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ContactParagraphPropertiesEditForm(BaseContactParagraphPropertiesEditForm):
    """Contact paragraph properties edit form"""

    fields = field.Fields(IContactParagraph).select('title', 'name', 'charge', 'photo', 'gps_location', 'city',
                                                    'address', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget


@view_config(name='properties.json', context=IContactParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, ContactParagraphPropertiesEditForm):
    """Contact properties edit form, JSON renderer"""


@adapter_config(context=(IContactParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class ContactParagraphInnerEditForm(ContactParagraphPropertiesEditForm):

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IContactParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ContactParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, ContactParagraphInnerEditForm):
    """Contact paragraph inner deit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(ContactParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IBaseParagraph, ())
        if 'title' in updated:
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                    'object_name': self.context.__name__,
                    'title': II18n(self.context).query_attribute('title', request=self.request),
                    'visible': self.context.visible
                }
            })
        updated = changes.get(IContactParagraphBase, ())
        if ('photo' in updated) or ('renderer' in updated):
            # we have to commit transaction to be able to handle blobs...
            ITransactionManager(self.context).get().commit()
            form = ContactParagraphInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'object_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'content': form.render()
                }
            })
        return output
